"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const core_1 = require("@ts-ioc/core");
const IRunnableBuilder_1 = require("./IRunnableBuilder");
const modules_1 = require("../modules");
const utils_1 = require("../utils");
const BootModule_1 = require("../BootModule");
const IConfigureManager_1 = require("./IConfigureManager");
const ConfigureRegister_1 = require("./ConfigureRegister");
const BootModuleInjector_1 = require("./BootModuleInjector");
/**
 * runnable events
 *
 * @export
 * @enum {number}
 */
var RunnableEvents;
(function (RunnableEvents) {
    /**
     * on root container created.
     */
    RunnableEvents["onRootContainerCreated"] = "onRootContainerCreated";
    /**
     * on root container inited.
     */
    RunnableEvents["onRootContainerInited"] = "onRootContainerInited";
    /**
     * on module created.
     */
    RunnableEvents["onModuleCreated"] = "onModuleCreated";
    /**
     * on registered runner use module.
     */
    RunnableEvents["registeredExt"] = "registeredExt";
})(RunnableEvents = exports.RunnableEvents || (exports.RunnableEvents = {}));
/**
 * runnable builder.
 *
 * @export
 * @class RunnableBuilder
 * @extends {ModuleBuilder}
 * @template T
 */
let RunnableBuilder = class RunnableBuilder extends modules_1.ModuleBuilder {
    constructor(baseURL) {
        super();
        this.inited = false;
        this._baseURL = baseURL;
        this.customRegs = [];
        this.depModules = [];
        this.beforeInitPds = new Map();
        this.afterInitPds = new Map();
        this.events = new utils_1.Events();
        this.initEvents();
    }
    initEvents() {
        this.on(RunnableEvents.onRootContainerInited, (container) => {
            this.afterInitPds.forEach((val, key) => {
                container.bindProvider(key, val);
            });
        });
    }
    on(name, event) {
        this.events.on(name, event);
        return this;
    }
    off(name, event) {
        this.events.off(name, event);
        return this;
    }
    emit(name, ...args) {
        this.events.emit(name, ...args);
    }
    getPools() {
        if (!this.pools) {
            this.pools = new utils_1.ContainerPool(this.createContainerBuilder());
            this.createDefaultContainer();
        }
        return this.pools;
    }
    getRunRoot(resolver) {
        return this._baseURL || (resolver || this.getPools().getDefault()).resolve(IRunnableBuilder_1.ProcessRunRootToken) || '';
    }
    /**
     * has register in pools.
     * use must after `initContainerPools`.
     *
     * @template T
     * @param {Token<T>} key
     * @returns {boolean}
     * @memberof RunnableBuilder
     */
    has(key, aliasOrway) {
        return this.getPools().values().some(c => c.has(key, aliasOrway));
    }
    /**
     * resove token in pools.
     * use must after `initContainerPools`.
     * @template T
     * @param {Token<T>} token
     * @param {...ParamProviders[]} providers
     * @returns {T}
     * @memberof RunnableBuilder
     */
    resolve(token, resway, ...providers) {
        let resolved;
        this.getPools().values().some(c => {
            resolved = c.resolve(token, resway, ...providers);
            return !core_1.isNullOrUndefined(resolved);
        });
        return resolved;
    }
    createContainerBuilder() {
        return new core_1.ContainerBuilder();
    }
    /**
     * use module as global Depdences module.
     *
     * @param {...LoadType[]} modules
     * @returns {this}
     * @memberof RunnableBuilder
     */
    use(...modules) {
        this.depModules = this.depModules.concat(modules);
        this.inited = false;
        return this;
    }
    /**
     * bind provider
     *
     * @template T
     * @param {Token<T>} provide
     * @param {Token<T> | Factory<T>} provider
     * @param {boolean} [beforeInit]
     * @returns {this}
     * @memberof RunnableBuilder
     */
    provider(provide, provider, beforeInit) {
        if (beforeInit) {
            this.beforeInitPds.set(provide, provider);
        }
        else {
            this.afterInitPds.set(provide, provider);
        }
        return this;
    }
    getProvider(provide, beforeInit) {
        if (beforeInit) {
            return this.beforeInitPds.has(provide) ? this.beforeInitPds.get(provide) : null;
        }
        else {
            return this.afterInitPds.has(provide) ? this.afterInitPds.get(provide) : null;
        }
    }
    initContainerPools() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            if (this.inited) {
                return;
            }
            let container = this.getPools().getDefault();
            yield this.registerExts(container);
            yield this.registerByConfigure(container);
            this.inited = true;
            this.events.emit(RunnableEvents.onRootContainerInited, container);
        });
    }
    load(token, config, options) {
        const _super = Object.create(null, {
            load: { get: () => super.load }
        });
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            yield this.initContainerPools();
            return yield _super.load.call(this, token, config, options);
        });
    }
    bootstrap(token, config, options) {
        const _super = Object.create(null, {
            bootstrap: { get: () => super.bootstrap }
        });
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            let params = this.vaildParams(token, config, options);
            options = params.options || {};
            let injmdl = params.token ? yield this.load(params.token, params.config, options) : yield this.load(params.config, options);
            options.env = injmdl;
            options.bootBuilder = this;
            options.configManager = this.getConfigManager();
            let builder = this.getBuilder(injmdl);
            if (!this.isSame(builder, this)) {
                return params.token ? yield builder.bootstrap(params.token, params.config, options) : yield builder.bootstrap(params.config, options);
            }
            else {
                return params.token ? yield _super.bootstrap.call(this, params.token, params.config, options) : yield _super.bootstrap.call(this, params.config, options);
            }
        });
    }
    isSame(mdb1, mdb2) {
        if (!mdb1 || !mdb2) {
            return false;
        }
        if (mdb1 === mdb2) {
            return true;
        }
        if (core_1.lang.getClass(mdb1) === core_1.lang.getClass(mdb2)) {
            return true;
        }
        return false;
    }
    /**
     * get module builder
     *
     * @param {(Token<T> |RunnableConfigure)} token
     * @param {ModuleEnv} [env]
     * @returns {IModuleBuilder<T>}
     * @memberof RunnableBuilder
     */
    getBuilderByConfig(token, options) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            let injmdl = yield this.load(token, options);
            return this.getBuilder(injmdl);
        });
    }
    getBuilder(injmdl) {
        let cfg = injmdl.config;
        let container = injmdl.container;
        let builder;
        if (cfg) {
            if (core_1.isToken(cfg.builder)) {
                if (core_1.isClass(cfg.builder)) {
                    if (!container.has(cfg.builder)) {
                        container.register(cfg.builder);
                    }
                    if (cfg.builder === core_1.lang.getClass(this)) {
                        builder = this;
                    }
                }
                if (!builder) {
                    builder = container.resolve(cfg.builder);
                }
            }
            else if (cfg.builder instanceof modules_1.ModuleBuilder) {
                builder = cfg.builder;
            }
        }
        let tko = injmdl.token;
        if (!builder && tko) {
            builder = container.getService(modules_1.ModuleBuilderToken, tko, (tk) => new modules_1.InjectModuleBuilderToken(tk), cfg.defaultBuilder || modules_1.ModuleBuilderToken);
        }
        return builder || this;
    }
    getConfigManager() {
        if (!this.configMgr) {
            this.configMgr = this.createConfigureMgr();
        }
        return this.configMgr;
    }
    createConfigureMgr() {
        let container = this.getPools().getDefault();
        return container.getService(IConfigureManager_1.ConfigureMgrToken, core_1.lang.getClass(this), { baseURL: this.getRunRoot(container) });
    }
    createDefaultContainer() {
        let container = this.pools.getDefault();
        container.register(BootModule_1.BootModule);
        let chain = container.getBuilder().getInjectorChain(container);
        chain.first(container.resolve(modules_1.DIModuleInjectorToken));
        chain.first(container.resolve(BootModuleInjector_1.BootstrapInjectorToken));
        container.bindProvider(utils_1.ContainerPoolToken, () => this.getPools());
        container.bindProvider(IRunnableBuilder_1.CurrentRunnableBuilderToken, () => this);
        this.beforeInitPds.forEach((val, key) => {
            container.bindProvider(key, val);
        });
        this.events.emit(RunnableEvents.onRootContainerCreated, container);
        return container;
    }
    /**
     * register ioc exts
     *
     * @protected
     * @param {IContainer} container
     * @param {AppConfigure} config
     * @memberof RunnableBuilder
     */
    registerExts(container) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            if (this.depModules.length) {
                let types = yield container.loadModule(...this.depModules);
                this.emit(RunnableEvents.registeredExt, types, container);
            }
        });
    }
    /**
     * register by configure.
     *
     * @protected
     * @param {IContainer} container
     * @param {ModuleConfig<T>} config
     * @returns {Promise<void>}
     * @memberof RunnableBuilder
     */
    registerByConfigure(container) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            let configManager = this.getConfigManager();
            let config = yield configManager.getConfig();
            if (!config.baseURL) {
                config.baseURL = this.getRunRoot(container);
            }
            yield core_1.PromiseUtil.step(this.customRegs.map(cs => () => tslib_1.__awaiter(this, void 0, void 0, function* () {
                let tokens = yield cs(container, config, this);
                return tokens;
            })));
            let curClass = core_1.lang.getClass(this);
            let registers = [];
            // only run root register.
            container.iteratorServices((serType, fac, resolver) => {
                if (!config.baseURL) {
                    config.baseURL = this.getRunRoot(resolver);
                }
                registers.push({
                    resolver: resolver,
                    serType: serType
                });
            }, ConfigureRegister_1.ConfigureRegister, curClass, true, core_1.ResoveWay.nodes);
            yield Promise.all(registers.map(ser => ser.resolver.resolve(ser.serType).register(config, this)));
        });
    }
    static getClassAnnations() {
        return { "name": "RunnableBuilder", "params": { "constructor": ["baseURL"], "initEvents": [], "on": ["name", "event"], "off": ["name", "event"], "emit": ["name", "args"], "getPools": [], "getRunRoot": ["resolver"], "has": ["key", "aliasOrway"], "resolve": ["token", "resway", "providers"], "createContainerBuilder": [], "use": ["modules"], "provider": ["provide", "provider", "beforeInit"], "getProvider": ["provide", "beforeInit"], "initContainerPools": [], "load": ["token", "config", "options"], "bootstrap": ["token", "config", "options"], "isSame": ["mdb1", "mdb2"], "getBuilderByConfig": ["token", "options"], "getBuilder": ["injmdl"], "getConfigManager": [], "createConfigureMgr": [], "createDefaultContainer": [], "registerExts": ["container"], "registerByConfigure": ["container"] } };
    }
};
RunnableBuilder = tslib_1.__decorate([
    core_1.Injectable(IRunnableBuilder_1.RunnableBuilderToken),
    tslib_1.__metadata("design:paramtypes", [String])
], RunnableBuilder);
exports.RunnableBuilder = RunnableBuilder;

//# sourceMappingURL=../sourcemaps/boot/RunnableBuilder.js.map
