"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
require("reflect-metadata");
const core_1 = require("@ts-ioc/core");
const IModuleBuilder_1 = require("./IModuleBuilder");
const utils_1 = require("../utils");
const annotations_1 = require("../annotations");
const InjectedModule_1 = require("./InjectedModule");
/**
 * inject module load token.
 *
 * @export
 * @class InjectModuleLoadToken
 * @extends {Registration<T>}
 * @template T
 */
class InjectModuleLoadToken extends core_1.Registration {
    constructor(token) {
        super(token, 'module_loader');
    }
    static getClassAnnations() {
        return { "name": "InjectModuleLoadToken", "params": { "constructor": ["token"] } };
    }
}
exports.InjectModuleLoadToken = InjectModuleLoadToken;
/**
 * module builder
 *
 * @export
 * @class ModuleBuilder
 * @implements {IModuleBuilder}
 * @template T
 */
let ModuleBuilder = class ModuleBuilder {
    constructor() {
    }
    getPools() {
        return this.pools;
    }
    getInjectedModule(type) {
        return this.getPools().getDefault().get(new InjectedModule_1.InjectedModuleToken(type));
    }
    /**
     * get metadata accessor.
     *
     * @param {IContainer} container
     * @param {Token<any>} token
     * @param {ModuleConfigure} [config]
     * @returns {IMetaAccessor<any>}
     * @memberof ModuleBuilder
     */
    getMetaAccessor(container, token, config) {
        let mtk;
        if (core_1.isToken(token)) {
            mtk = token;
        }
        else {
            config = token;
        }
        return container.getService(core_1.MetaAccessorToken, mtk ? [mtk, core_1.lang.getClass(this)] : core_1.lang.getClass(this), tk => new core_1.InjectMetaAccessorToken(tk), config ? (config.defaultMetaAccessor || core_1.MetaAccessorToken) : core_1.MetaAccessorToken);
    }
    /**
     * load module.
     *
     * @param {(Token<T> | ModuleConfigure)} token
     * @param {(ModuleConfig<T> | BootOptions<T>)} [config]
     * @param {BootOptions<T>} [options]
     * @returns {Promise<InjectedModule<T>>}
     * @memberof ModuleBuilder
     */
    load(token, config, options) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            let params = this.vaildParams(token, config, options);
            let env = params.options ? params.options.env : null;
            if (env instanceof InjectedModule_1.InjectedModule) {
                return env;
            }
            let parent = yield this.getParentContainer(env);
            if (params.token) {
                let injmd = yield this.loadViaToken(params.token, parent);
                if (params.config) {
                    yield this.loadViaConfig(params.config, injmd.container || parent);
                }
                return injmd;
            }
            else {
                return yield this.loadViaConfig(params.config, parent);
            }
        });
    }
    /**
     * bootstrap with module and config.
     *
     * @param {(Token<T> | ModuleConfig<T>)} token
     * @param {(ModuleConfig<T> | BootOptions<T>)} [config]
     * @param {BootOptions<T>} [options]
     * @returns {Promise<Runnable<T>>}
     * @memberof ModuleBuilder
     */
    bootstrap(token, config, options) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            let params = this.vaildParams(token, config, options);
            options = params.options || {};
            let injmdl;
            if (params.token) {
                injmdl = yield this.load(params.token, params.config, options);
            }
            else {
                injmdl = yield this.load(params.config, options);
            }
            options.env = injmdl;
            let cfg = injmdl.config;
            let tk = injmdl.token || injmdl.type;
            let container = injmdl.container;
            let accessor = this.getMetaAccessor(container, tk);
            let bootToken = accessor.getBootToken(cfg, container);
            if (bootToken) {
                let anBuilder = this.getAnnoBuilder(container, bootToken, cfg);
                return yield anBuilder.boot(bootToken, cfg, options);
            }
            else {
                let mdBuilder = this.getAnnoBuilder(container, tk, cfg);
                return yield mdBuilder.boot(tk, cfg, options);
            }
        });
    }
    vaildParams(token, config, options) {
        let params = {};
        if (core_1.isToken(token)) {
            params.token = token;
            if (core_1.isUndefined(options)) {
                params.options = config;
            }
            else {
                params.config = config;
                params.options = options;
            }
        }
        else {
            params.config = token;
            params.options = config;
        }
        return params;
    }
    loadViaToken(token, parent) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            let injmdl = yield this.import(token, parent);
            if (!injmdl) {
                let cfg = this.getMetaAccessor(parent, token).getMetadata(token, parent);
                injmdl = new InjectedModule_1.InjectedModule(token, cfg, parent);
            }
            return injmdl;
        });
    }
    loadViaConfig(config, parent) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            let injmd = null;
            let token = this.getMetaAccessor(parent, config).getToken(config, parent);
            if (token) {
                injmd = yield this.import(token, parent);
                if (!injmd) {
                    let cfg = this.getMetaAccessor(parent, token).getMetadata(token, parent, config);
                    injmd = new InjectedModule_1.InjectedModule(token, cfg, parent);
                }
            }
            else {
                token = null;
            }
            if (core_1.isArray(config.imports) && config.imports.length) {
                yield parent.loadModule(...config.imports);
            }
            if (core_1.isArray(config.providers) && config.providers.length) {
                let parser = parent.getProviderParser();
                let pdrmap = parser.parse(...config.providers);
                pdrmap.keys().forEach(key => {
                    core_1.isToken(key) && parent.bindProvider(key, (...providers) => pdrmap.resolve(key, ...providers));
                });
            }
            if (injmd) {
                injmd.config = core_1.lang.assign({}, injmd.config, config);
            }
            else {
                injmd = new InjectedModule_1.InjectedModule(token, config, parent);
            }
            return injmd;
        });
    }
    import(token, parent) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            let type = core_1.isClass(token) ? token : parent.getTokenImpl(token);
            if (core_1.isClass(type)) {
                let key = new InjectedModule_1.InjectedModuleToken(type);
                if (parent.has(key)) {
                    return parent.get(key);
                }
                else {
                    yield parent.loadModule(type);
                    if (parent.has(key)) {
                        return parent.get(key);
                    }
                }
            }
            return null;
        });
    }
    getParentContainer(env) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            let parent;
            if (env) {
                if (env instanceof core_1.Container) {
                    parent = env;
                }
                else if (env instanceof InjectedModule_1.InjectedModule) {
                    parent = env.container.parent;
                }
            }
            if (!parent) {
                parent = this.getPools().getDefault();
            }
            return parent;
        });
    }
    getAnnoBuilder(container, token, config) {
        let builder;
        if (core_1.isClass(config.annoBuilder)) {
            if (!container.has(config.annoBuilder)) {
                container.register(config.annoBuilder);
            }
        }
        if (core_1.isToken(config.annoBuilder)) {
            builder = container.resolve(config.annoBuilder);
        }
        else if (config.annoBuilder instanceof annotations_1.AnnotationBuilder) {
            builder = config.annoBuilder;
        }
        if (!builder && token) {
            builder = container.getService(annotations_1.AnnotationBuilderToken, token, tk => new annotations_1.InjectAnnotationBuilder(tk), config.defaultAnnoBuilder || annotations_1.AnnotationBuilderToken);
        }
        if (builder) {
            builder.container = container;
        }
        return builder;
    }
    static getClassAnnations() {
        return { "name": "ModuleBuilder", "params": { "constructor": [], "getPools": [], "getInjectedModule": ["type"], "getMetaAccessor": ["container", "token", "config"], "load": ["token", "config", "options"], "bootstrap": ["token", "config", "options"], "vaildParams": ["token", "config", "options"], "loadViaToken": ["token", "parent"], "loadViaConfig": ["config", "parent"], "import": ["token", "parent"], "getParentContainer": ["env"], "getAnnoBuilder": ["container", "token", "config"] } };
    }
};
tslib_1.__decorate([
    core_1.Inject(utils_1.ContainerPoolToken),
    tslib_1.__metadata("design:type", utils_1.ContainerPool)
], ModuleBuilder.prototype, "pools", void 0);
ModuleBuilder = tslib_1.__decorate([
    core_1.Singleton(IModuleBuilder_1.ModuleBuilderToken),
    tslib_1.__metadata("design:paramtypes", [])
], ModuleBuilder);
exports.ModuleBuilder = ModuleBuilder;

//# sourceMappingURL=../sourcemaps/modules/ModuleBuilder.js.map
