"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var core_1 = require("@ts-ioc/core");
var IRunnableBuilder_1 = require("./IRunnableBuilder");
var modules_1 = require("../modules");
var utils_1 = require("../utils");
var BootModule_1 = require("../BootModule");
var IConfigureManager_1 = require("./IConfigureManager");
var ConfigureRegister_1 = require("./ConfigureRegister");
var BootModuleInjector_1 = require("./BootModuleInjector");
/**
 * runnable events
 *
 * @export
 * @enum {number}
 */
var RunnableEvents;
(function (RunnableEvents) {
    /**
     * on root container created.
     */
    RunnableEvents["onRootContainerCreated"] = "onRootContainerCreated";
    /**
     * on root container inited.
     */
    RunnableEvents["onRootContainerInited"] = "onRootContainerInited";
    /**
     * on module created.
     */
    RunnableEvents["onModuleCreated"] = "onModuleCreated";
    /**
     * on registered runner use module.
     */
    RunnableEvents["registeredExt"] = "registeredExt";
})(RunnableEvents = exports.RunnableEvents || (exports.RunnableEvents = {}));
/**
 * runnable builder.
 *
 * @export
 * @class RunnableBuilder
 * @extends {ModuleBuilder}
 * @template T
 */
var RunnableBuilder = /** @class */ (function (_super) {
    tslib_1.__extends(RunnableBuilder, _super);
    function RunnableBuilder(baseURL) {
        var _this = _super.call(this) || this;
        _this.inited = false;
        _this._baseURL = baseURL;
        _this.customRegs = [];
        _this.depModules = [];
        _this.beforeInitPds = new Map();
        _this.afterInitPds = new Map();
        _this.events = new utils_1.Events();
        _this.initEvents();
        return _this;
    }
    RunnableBuilder.prototype.initEvents = function () {
        var _this = this;
        this.on(RunnableEvents.onRootContainerInited, function (container) {
            _this.afterInitPds.forEach(function (val, key) {
                container.bindProvider(key, val);
            });
        });
    };
    RunnableBuilder.prototype.on = function (name, event) {
        this.events.on(name, event);
        return this;
    };
    RunnableBuilder.prototype.off = function (name, event) {
        this.events.off(name, event);
        return this;
    };
    RunnableBuilder.prototype.emit = function (name) {
        var args = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            args[_i - 1] = arguments[_i];
        }
        var _a;
        (_a = this.events).emit.apply(_a, [name].concat(args));
    };
    RunnableBuilder.prototype.getPools = function () {
        if (!this.pools) {
            this.pools = new utils_1.ContainerPool(this.createContainerBuilder());
            this.createDefaultContainer();
        }
        return this.pools;
    };
    RunnableBuilder.prototype.getRunRoot = function (resolver) {
        return this._baseURL || (resolver || this.getPools().getDefault()).resolve(IRunnableBuilder_1.ProcessRunRootToken) || '';
    };
    /**
     * has register in pools.
     * use must after `initContainerPools`.
     *
     * @template T
     * @param {Token<T>} key
     * @returns {boolean}
     * @memberof RunnableBuilder
     */
    RunnableBuilder.prototype.has = function (key, aliasOrway) {
        return this.getPools().values().some(function (c) { return c.has(key, aliasOrway); });
    };
    /**
     * resove token in pools.
     * use must after `initContainerPools`.
     * @template T
     * @param {Token<T>} token
     * @param {...ParamProviders[]} providers
     * @returns {T}
     * @memberof RunnableBuilder
     */
    RunnableBuilder.prototype.resolve = function (token, resway) {
        var providers = [];
        for (var _i = 2; _i < arguments.length; _i++) {
            providers[_i - 2] = arguments[_i];
        }
        var resolved;
        this.getPools().values().some(function (c) {
            resolved = c.resolve.apply(c, [token, resway].concat(providers));
            return !core_1.isNullOrUndefined(resolved);
        });
        return resolved;
    };
    RunnableBuilder.prototype.createContainerBuilder = function () {
        return new core_1.ContainerBuilder();
    };
    /**
     * use module as global Depdences module.
     *
     * @param {...LoadType[]} modules
     * @returns {this}
     * @memberof RunnableBuilder
     */
    RunnableBuilder.prototype.use = function () {
        var modules = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            modules[_i] = arguments[_i];
        }
        this.depModules = this.depModules.concat(modules);
        this.inited = false;
        return this;
    };
    /**
     * bind provider
     *
     * @template T
     * @param {Token<T>} provide
     * @param {Token<T> | Factory<T>} provider
     * @param {boolean} [beforeInit]
     * @returns {this}
     * @memberof RunnableBuilder
     */
    RunnableBuilder.prototype.provider = function (provide, provider, beforeInit) {
        if (beforeInit) {
            this.beforeInitPds.set(provide, provider);
        }
        else {
            this.afterInitPds.set(provide, provider);
        }
        return this;
    };
    RunnableBuilder.prototype.getProvider = function (provide, beforeInit) {
        if (beforeInit) {
            return this.beforeInitPds.has(provide) ? this.beforeInitPds.get(provide) : null;
        }
        else {
            return this.afterInitPds.has(provide) ? this.afterInitPds.get(provide) : null;
        }
    };
    RunnableBuilder.prototype.initContainerPools = function () {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var container;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (this.inited) {
                            return [2 /*return*/];
                        }
                        container = this.getPools().getDefault();
                        return [4 /*yield*/, this.registerExts(container)];
                    case 1:
                        _a.sent();
                        return [4 /*yield*/, this.registerByConfigure(container)];
                    case 2:
                        _a.sent();
                        this.inited = true;
                        this.events.emit(RunnableEvents.onRootContainerInited, container);
                        return [2 /*return*/];
                }
            });
        });
    };
    RunnableBuilder.prototype.load = function (token, config, options) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.initContainerPools()];
                    case 1:
                        _a.sent();
                        return [4 /*yield*/, _super.prototype.load.call(this, token, config, options)];
                    case 2: return [2 /*return*/, _a.sent()];
                }
            });
        });
    };
    RunnableBuilder.prototype.bootstrap = function (token, config, options) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var params, injmdl, _a, builder, _b, _c;
            return tslib_1.__generator(this, function (_d) {
                switch (_d.label) {
                    case 0:
                        params = this.vaildParams(token, config, options);
                        options = params.options || {};
                        if (!params.token) return [3 /*break*/, 2];
                        return [4 /*yield*/, this.load(params.token, params.config, options)];
                    case 1:
                        _a = _d.sent();
                        return [3 /*break*/, 4];
                    case 2: return [4 /*yield*/, this.load(params.config, options)];
                    case 3:
                        _a = _d.sent();
                        _d.label = 4;
                    case 4:
                        injmdl = _a;
                        options.env = injmdl;
                        options.bootBuilder = this;
                        options.configManager = this.getConfigManager();
                        builder = this.getBuilder(injmdl);
                        if (!!this.isSame(builder, this)) return [3 /*break*/, 9];
                        if (!params.token) return [3 /*break*/, 6];
                        return [4 /*yield*/, builder.bootstrap(params.token, params.config, options)];
                    case 5:
                        _b = _d.sent();
                        return [3 /*break*/, 8];
                    case 6: return [4 /*yield*/, builder.bootstrap(params.config, options)];
                    case 7:
                        _b = _d.sent();
                        _d.label = 8;
                    case 8: return [2 /*return*/, _b];
                    case 9:
                        if (!params.token) return [3 /*break*/, 11];
                        return [4 /*yield*/, _super.prototype.bootstrap.call(this, params.token, params.config, options)];
                    case 10:
                        _c = _d.sent();
                        return [3 /*break*/, 13];
                    case 11: return [4 /*yield*/, _super.prototype.bootstrap.call(this, params.config, options)];
                    case 12:
                        _c = _d.sent();
                        _d.label = 13;
                    case 13: return [2 /*return*/, _c];
                }
            });
        });
    };
    RunnableBuilder.prototype.isSame = function (mdb1, mdb2) {
        if (!mdb1 || !mdb2) {
            return false;
        }
        if (mdb1 === mdb2) {
            return true;
        }
        if (core_1.lang.getClass(mdb1) === core_1.lang.getClass(mdb2)) {
            return true;
        }
        return false;
    };
    /**
     * get module builder
     *
     * @param {(Token<T> |RunnableConfigure)} token
     * @param {ModuleEnv} [env]
     * @returns {IModuleBuilder<T>}
     * @memberof RunnableBuilder
     */
    RunnableBuilder.prototype.getBuilderByConfig = function (token, options) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var injmdl;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.load(token, options)];
                    case 1:
                        injmdl = _a.sent();
                        return [2 /*return*/, this.getBuilder(injmdl)];
                }
            });
        });
    };
    RunnableBuilder.prototype.getBuilder = function (injmdl) {
        var cfg = injmdl.config;
        var container = injmdl.container;
        var builder;
        if (cfg) {
            if (core_1.isToken(cfg.builder)) {
                if (core_1.isClass(cfg.builder)) {
                    if (!container.has(cfg.builder)) {
                        container.register(cfg.builder);
                    }
                    if (cfg.builder === core_1.lang.getClass(this)) {
                        builder = this;
                    }
                }
                if (!builder) {
                    builder = container.resolve(cfg.builder);
                }
            }
            else if (cfg.builder instanceof modules_1.ModuleBuilder) {
                builder = cfg.builder;
            }
        }
        var tko = injmdl.token;
        if (!builder && tko) {
            builder = container.getService(modules_1.ModuleBuilderToken, tko, function (tk) { return new modules_1.InjectModuleBuilderToken(tk); }, cfg.defaultBuilder || modules_1.ModuleBuilderToken);
        }
        return builder || this;
    };
    RunnableBuilder.prototype.getConfigManager = function () {
        if (!this.configMgr) {
            this.configMgr = this.createConfigureMgr();
        }
        return this.configMgr;
    };
    RunnableBuilder.prototype.createConfigureMgr = function () {
        var container = this.getPools().getDefault();
        return container.getService(IConfigureManager_1.ConfigureMgrToken, core_1.lang.getClass(this), { baseURL: this.getRunRoot(container) });
    };
    RunnableBuilder.prototype.createDefaultContainer = function () {
        var _this = this;
        var container = this.pools.getDefault();
        container.register(BootModule_1.BootModule);
        var chain = container.getBuilder().getInjectorChain(container);
        chain.first(container.resolve(modules_1.DIModuleInjectorToken));
        chain.first(container.resolve(BootModuleInjector_1.BootstrapInjectorToken));
        container.bindProvider(utils_1.ContainerPoolToken, function () { return _this.getPools(); });
        container.bindProvider(IRunnableBuilder_1.CurrentRunnableBuilderToken, function () { return _this; });
        this.beforeInitPds.forEach(function (val, key) {
            container.bindProvider(key, val);
        });
        this.events.emit(RunnableEvents.onRootContainerCreated, container);
        return container;
    };
    /**
     * register ioc exts
     *
     * @protected
     * @param {IContainer} container
     * @param {AppConfigure} config
     * @memberof RunnableBuilder
     */
    RunnableBuilder.prototype.registerExts = function (container) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var types;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!this.depModules.length) return [3 /*break*/, 2];
                        return [4 /*yield*/, container.loadModule.apply(container, this.depModules)];
                    case 1:
                        types = _a.sent();
                        this.emit(RunnableEvents.registeredExt, types, container);
                        _a.label = 2;
                    case 2: return [2 /*return*/];
                }
            });
        });
    };
    /**
     * register by configure.
     *
     * @protected
     * @param {IContainer} container
     * @param {ModuleConfig<T>} config
     * @returns {Promise<void>}
     * @memberof RunnableBuilder
     */
    RunnableBuilder.prototype.registerByConfigure = function (container) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var configManager, config, curClass, registers;
            var _this = this;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        configManager = this.getConfigManager();
                        return [4 /*yield*/, configManager.getConfig()];
                    case 1:
                        config = _a.sent();
                        if (!config.baseURL) {
                            config.baseURL = this.getRunRoot(container);
                        }
                        return [4 /*yield*/, core_1.PromiseUtil.step(this.customRegs.map(function (cs) { return function () { return tslib_1.__awaiter(_this, void 0, void 0, function () {
                                var tokens;
                                return tslib_1.__generator(this, function (_a) {
                                    switch (_a.label) {
                                        case 0: return [4 /*yield*/, cs(container, config, this)];
                                        case 1:
                                            tokens = _a.sent();
                                            return [2 /*return*/, tokens];
                                    }
                                });
                            }); }; }))];
                    case 2:
                        _a.sent();
                        curClass = core_1.lang.getClass(this);
                        registers = [];
                        // only run root register.
                        container.iteratorServices(function (serType, fac, resolver) {
                            if (!config.baseURL) {
                                config.baseURL = _this.getRunRoot(resolver);
                            }
                            registers.push({
                                resolver: resolver,
                                serType: serType
                            });
                        }, ConfigureRegister_1.ConfigureRegister, curClass, true, core_1.ResoveWay.nodes);
                        return [4 /*yield*/, Promise.all(registers.map(function (ser) { return ser.resolver.resolve(ser.serType).register(config, _this); }))];
                    case 3:
                        _a.sent();
                        return [2 /*return*/];
                }
            });
        });
    };
    RunnableBuilder.getClassAnnations = function () {
        return { "name": "RunnableBuilder", "params": { "constructor": ["baseURL"], "initEvents": [], "on": ["name", "event"], "off": ["name", "event"], "emit": ["name", "args"], "getPools": [], "getRunRoot": ["resolver"], "has": ["key", "aliasOrway"], "resolve": ["token", "resway", "providers"], "createContainerBuilder": [], "use": ["modules"], "provider": ["provide", "provider", "beforeInit"], "getProvider": ["provide", "beforeInit"], "initContainerPools": [], "load": ["token", "config", "options"], "bootstrap": ["token", "config", "options"], "isSame": ["mdb1", "mdb2"], "getBuilderByConfig": ["token", "options"], "getBuilder": ["injmdl"], "getConfigManager": [], "createConfigureMgr": [], "createDefaultContainer": [], "registerExts": ["container"], "registerByConfigure": ["container"] } };
    };
    RunnableBuilder = tslib_1.__decorate([
        core_1.Injectable(IRunnableBuilder_1.RunnableBuilderToken),
        tslib_1.__metadata("design:paramtypes", [String])
    ], RunnableBuilder);
    return RunnableBuilder;
}(modules_1.ModuleBuilder));
exports.RunnableBuilder = RunnableBuilder;

//# sourceMappingURL=../sourcemaps/boot/RunnableBuilder.js.map
